<?php

if (!class_exists("WC_Payerurl")) {

    class WC_Payerurl extends WC_Payment_Gateway
    {
        public function __construct()
        {
            $this->id = 'wc_payerurl_gateway';
            $this->icon = PAYERURL_URL . 'assets/images/coin_high.png';
            $this->method_title = 'Payerurl';
            $this->method_description = '';
            $this->has_fields = false;

            $this->init_form_fields();
            $this->init_settings();

            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->payerurl_public_key = $this->get_option('payerurl_public_key');
            $this->payerurl_secret_key = $this->get_option('payerurl_secret_key');

            if (is_admin()) {
                add_action(
                    'woocommerce_update_options_payment_gateways_' . $this->id,
                    array($this, 'process_admin_options')
                );
            }
            add_action('woocommerce_api_wc_payerurl', array($this, 'payerurl_response'));
        }

        public function init_form_fields()
        {
            $this->form_fields = array(
                'title' => array(
                    'title' => __('Title', 'payerurl-crypto-currency-payment-gateway-for-wooCommerce'),
                    'type' => 'text',
                    'default' => __('Payerurl', 'payerurl-crypto-currency-payment-gateway-for-wooCommerce'),
                ),
                'description' => array(
                    'title' => __('Description', 'payerurl-crypto-currency-payment-gateway-for-wooCommerce'),
                    'type' => 'textarea',
                    'default' => __(
                        'Pay securely using Payerurl',
                        'payerurl-crypto-currency-payment-gateway-for-wooCommerce'
                    ),
                ),
                'payerurl_public_key' => array(
                    'title' => __('Public key', 'payerurl-crypto-currency-payment-gateway-for-wooCommerce'),
                    'type' => 'password',
                ),
                'payerurl_secret_key' => array(
                    'title' => __('Secret key', 'payerurl-crypto-currency-payment-gateway-for-wooCommerce'),
                    'type' => 'password',
                ),
            );
        }

        public function payment_fields()
        {
            if ($this->description) echo esc_html(wptexturize($this->description));
        }

        public function process_admin_options()
        {
            $saved = parent::process_admin_options();
            return $saved;
        }

        public function process_payment($order_id)
        {
            global $woocommerce;
            $order = new WC_Order($order_id);
            $args = [
                'timeout' => 45,
                'redirection' => 5,
                'body' => $this->getRequestBody($order)
            ];
            $signature = $this->generateSignature($args['body'], $this->payerurl_secret_key);
            $authStr = base64_encode(sprintf('%s:%s', $this->payerurl_public_key, $signature));
            $args['headers'] = [
                'Content-Type' => 'application/x-www-form-urlencoded;charset=UTF-8',
                'Authorization' => sprintf('Bearer %s', $authStr),
            ];

            $response = wp_remote_post(esc_url_raw(PAYERURL_PAYMENT_URL), $args);
            if (!is_wp_error($response)) {
                $body = json_decode($response['body'], true);
                if (isset($body['redirectTO'])) {
                    $woocommerce->cart->empty_cart();
                    return array(
                        'result' => 'success',
                        'redirect' => esc_url_raw($body['redirectTO'])
                    );
                }
            }

            wc_add_notice(
                __(
                    'An error occurred, We were unable to process your order, please try again.',
                    'payerurl-crypto-currency-payment-gateway-for-wooCommerce'
                ),
                'error'
            );
            return array('result' => 'error', 'redirect' => wc_get_checkout_url());
        }

        public function payerurl_response()
        {
            if (!isset($_SERVER['HTTP_AUTHORIZATION']) || empty($_SERVER['HTTP_AUTHORIZATION'])) return;

            $authStr = str_replace('Bearer ', '', $_SERVER['HTTP_AUTHORIZATION']);
            $authStr = base64_decode($authStr);
            $auth = explode(':', $authStr);

            if ($this->payerurl_public_key != $auth[0]) return;
            if (!isset($_POST['transaction_id']) || empty($_POST['transaction_id'])) return;
            if (!isset($_POST['order_id']) || empty($_POST['order_id'])) return;
            $order = wc_get_order($_POST['order_id']);
            if (is_null($order)) return;

            $input = [
                'order_id' => $order->get_id(),
                'ext_transaction_id' => $this->getDataFromResponse('ext_transaction_id'),
                'transaction_id' => sanitize_text_field($_POST['transaction_id']),
                'status_code' => intval($this->getDataFromResponse('status_code')),
                'note' => $this->getDataFromResponse('note'),
                'confirm_rcv_amnt' => $this->getDataFromResponse('confirm_rcv_amnt', 0),
                'confirm_rcv_amnt_curr' => $this->getDataFromResponse('confirm_rcv_amnt_curr'),
                'coin_rcv_amnt' => $this->getDataFromResponse('coin_rcv_amnt', 0),
                'coin_rcv_amnt_curr' => $this->getDataFromResponse('coin_rcv_amnt_curr'),
            ];

            $signature = $this->generateSignature($input, $this->payerurl_secret_key);
            if ($signature != $auth[1]) return;

            $input['confirm_rcv_amnt'] = round($input['confirm_rcv_amnt'], 2);
            $input['confirm_rcv_amnt_curr'] = strtoupper($input['confirm_rcv_amnt_curr']);
            $link = filter_var(
                'https://tronscan.org/#/transaction/' . $input['ext_transaction_id'],
                FILTER_SANITIZE_URL
            );

            $format_link = sprintf(
                '<a href="%s" target="_blank" rel="noopener">%s</a>',
                $link,
                $input['ext_transaction_id']
            );

            $order->add_order_note(
                sprintf(
                    'Txn ID: %s<br/>Txn. Hash: %s<br/>Received Coin: %s %s<br/>
                    Received Amount: %s %s<br/>Note: %s',
                    $input['transaction_id'],
                    $format_link,
                    $input['coin_rcv_amnt'],
                    $input['coin_rcv_amnt_curr'],
                    $input['confirm_rcv_amnt'],
                    $input['confirm_rcv_amnt_curr'],
                    $input['note'],
                ),
                true
            );
            if (
                $input['status_code'] === 200 &&
                $input['confirm_rcv_amnt'] != 0 && $input['coin_rcv_amnt'] != 0 &&
                $input['confirm_rcv_amnt'] == $order->get_total()
            ) {
                $order->payment_complete($input['transaction_id']);
            } else {
                $order->set_transaction_id($input['transaction_id']);
                $order->update_status('on-hold');
                $order->save();
            }
        }

        private function generateSignature($input, $key)
        {
            ksort($input);
            $input = array_map(function ($value) {
                return $value !== false && empty($value) ? null : $value;
            }, $input);
            $input = http_build_query($input);
            return hash_hmac('sha256', $input, $key);
        }

        private function getRequestBody($order)
        {
            return array(
                'order_id' => $order->get_id(),
                'order_key' => $order->get_order_key(),
                'amount' => $order->get_total(),
                'currency' => strtolower($order->get_currency()),
                'billing_fname' => sanitize_text_field($order->get_billing_first_name()),
                'billing_lname' => sanitize_text_field($order->get_billing_last_name()),
                'billing_email' => sanitize_email($order->get_billing_email()),
                'redirect_to' => $order->get_checkout_order_received_url(),
                'type' => 'wp',
                'notify_url' => home_url('/wc-api/wc_payerurl')
            );
        }

        private function getDataFromResponse($key, $default = '')
        {
            return isset($_POST[$key]) ? sanitize_text_field($_POST[$key]) : $default;
        }
    }
}
